! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
!! Module: efftplot_mod 
!
! Authors:  Natalie Webb - Didier Barret - Jean-Francois Olive - CESR
!          
! Description: PERFORMS FFT OF EPIC TIME-SERIES
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module efftplot_mod

  use dal
  use param  
  use metatask_mod
  use dss
  use sla
#ifdef NAGf90Fortran
  use f90_unix_proc
#endif

  implicit none 
  
  private 

  public efftplot

contains  
  
!---------------------
    subroutine efftplot
!---------------------
     
    integer,parameter :: nb_options = 36
    integer,parameter :: nbusrpara  = 13
    integer,dimension(:),pointer :: l_nb_sub_col
    integer :: nb_pt, nb_col, col, subcol, nbopt, maxopt, Size_Vectors
    integer :: var1, nrows, nrow_gti
    character(len=128) :: Input_File,Output_File, GTI_File
    character(len=1000),dimension(8):: Label
    character(len=10 ),dimension(6) :: l_name_col
    character(len=8) :: powspec = 'powspec '
    character(len=3) :: rm = 'rm '
    character(len=128) :: opt1
    character(len=512) :: Commande,opt2
    character(len=15) :: Device_type
    character(len=15),dimension(2,nb_options) :: listoptions
    character(len=1) :: c1,v1
    character(len=12) :: backapp, INSTRUME
    character(len=6) :: fcopy = 'fcopy '
    character(len=512) :: Command, commgtis
    character(len=15) :: gti2xronwin = 'gti2xronwin -i '
    logical :: nogtis

    real(kind=double),dimension(:),pointer :: Freq  
    real(kind=single),dimension(:),pointer :: Pds, Pds_Err 
    real(kind=single),dimension(:,:),pointer :: Array_Rate, Array_Error,Bkg_Err
    real(kind=single),dimension(:,:),pointer :: Array_Backv, Array_Backe
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Err, Bkg_Rate
  
    type( DataSetT ) :: Set, Set_temp, Set_gti
    type( TableT ) :: Table1, Table1_temp, Table_gti
    type( ColumnT ) :: Col_Rate , Col_Error , Col_Backv , Col_Backe, Col_Start

    maxopt = size(listoptions,2)
    label(1) = 'Label 1'
    label(2) = 'Label 2'
    label(3) = 'Label 3'
    label(4) = 'Label 4'

!If the net count is provided, efftplot calculates the total count

    Input_File = StringParameter("infile")
    Set = dataSet(Input_File,READ)
    Table1 = table(Set,BlockNumber(Set,'RATE')) 

    call create_labels(Input_File,Set,Label,INSTRUME)

    if (HasAttribute(Set,"BACKAPP") ) then
         backapp = StringAttribute(Set,'BACKAPP')
    else if (HasAttribute(Table1,"BACKAPP") ) then
         backapp = StringAttribute(Table1,'BACKAPP')
    else
         backapp = 'F'
    endif

! Reads the file if it is NOT background subtracted.  Also carries out the 
! background subtraction if it is required (given with the bkgsub parameter).

    if (trim(backapp) == 'F') then 

    if (booleanParameter("bkgsub")) then
      Command = fcopy // ' "'// trim(Input_File) // '" '// ' "!temp.fit"'
      call system(Command) 
      Set_temp = dataSet("temp.fit",modify)
      Table1_temp = table(Set_temp,'RATE',read) 
      Col_Rate = Column(Table1_temp,'RATE',Modify)
      Col_Error = Column(Table1_temp,'ERROR',Modify)
    end if

    if (.not. booleanParameter("bkgsub")) then
      Col_Rate = Column(Table1,'RATE',read)
      Col_Error = Column(Table1,'ERROR',read)
    end if

      Col_Backv = Column(Table1,'BACKV',read)
      Col_Backe = Column(Table1,'BACKE',read)

      nrows = numberOfRows(Col_Rate)

      Array_Rate => real32VectorData(Col_Rate)
      Array_Error => real32VectorData(Col_Error)
      Array_Backv => real32VectorData(Col_Backv)
      Array_Backe => real32VectorData(Col_Backe)

      allocate(Net_Rate(size(Array_Rate(:,0)), size(Array_Rate(0,:)) ) )
      allocate(Net_Err(size(Array_Error(:,0)), size(Array_Error(0,:)) ) )
      allocate(Bkg_Rate(size(Array_Backv(:,0)), size(Array_Backv(0,:)) ) )
      allocate(Bkg_Err(size(Array_Backe(:,0)), size(Array_Backe(0,:)) ) )

      Net_Rate = Array_Rate 
      Net_Err = Array_Error
      Bkg_Rate = Array_Backv ; Bkg_Err = Array_Backe
      
      if (minval(Net_Rate)  < 0.0 ) &
         call error('ELCPLOT','Total rate negative')
      if (minval(Bkg_Rate)  < 0.0 ) &
         call error('ELCPLOT','Background rate negative')

! You should note that although the count rates are called NET - they are in 
! fact the NON-background subtracted rates.

      if (booleanParameter("bkgsub")) then
        Array_Rate = Net_Rate-Bkg_Rate
!This is negtive because the net counts are correlated with the total and the
!background counts
        Array_Error = sqrt(Net_Err**2.0+Bkg_Err**2.0)
      end if

      deallocate(Net_Rate, Net_Err, Bkg_Rate, Bkg_Err)
      call release(Col_Rate)
      call release(Col_Error)
      call release(Col_Backv)
      call release(Col_Backe)

      if (booleanParameter("bkgsub")) then
        call release(Table1_temp)
        call release(Set_temp)
        Input_File="temp.fit"
      end if
    end if

! Reads the file if it IS background subtracted.  

    if (trim(backapp) == 'T') then 
      Command = fcopy // ' "'// trim(Input_File) // '" '// ' "!temp.fit"'
      call system(Command) 
      Set_temp = dataSet("temp.fit",modify)
      Table1_temp = table(Set_temp,'RATE',read) 
      Col_Rate = Column(Table1_temp,'RATE',Modify)
      Col_Error = Column(Table1_temp,'ERROR',Modify)
      Col_Backv = Column(Table1_temp,'BACKV',read)
      Col_Backe = Column(Table1_temp,'BACKE',read)

      nrows = numberOfRows(Col_Rate)

      Array_Rate => real32VectorData(Col_Rate)
      Array_Error => real32VectorData(Col_Error)
      Array_Backv => real32VectorData(Col_Backv)
      Array_Backe => real32VectorData(Col_Backe)

      allocate(Net_Rate(size(Array_Rate(:,0)), size(Array_Rate(0,:)) ) )
      allocate(Net_Err(size(Array_Error(:,0)), size(Array_Error(0,:)) ) )
      allocate(Bkg_Rate(size(Array_Backv(:,0)), size(Array_Backv(0,:)) ) )
      allocate(Bkg_Err(size(Array_Backe(:,0)), size(Array_Backe(0,:)) ) )

      Net_Rate = Array_Rate 
      Net_Err = Array_Error
      Bkg_Rate = Array_Backv ; Bkg_Err = Array_Backe
      
      if (minval(Bkg_Rate)  < 0.0 ) &
         call error('ELCPLOT','Background rate negative')

! You should note that although the count rates are called NET - they are in 
! fact the NON-background subtracted rates.

    if (.not. booleanParameter("bkgsub")) then
      Array_Rate = Net_Rate+Bkg_Rate
!This is negative because the net counts are correlated with the total and the
!background counts
      Array_Error = sqrt(Net_Err**2.0-Bkg_Err**2.0)
      if (minval(Array_Rate)  < 0.0 ) &
         call error('ELCPLOT','Total rate negative')
      if (minval(Array_Error)  < 0.0 ) &
         call error('ELCPLOT','Rate error negative')
     end if

      deallocate(Net_Rate, Net_Err, Bkg_Rate, Bkg_Err)
      call release(Col_Rate)
      call release(Col_Error)
      call release(Col_Backv)
      call release(Col_Backe)
      call release(Table1_temp)
      call release(Set_temp)
      Input_File="temp.fit"
    end if

! efftplot now takes the user provided GTI file if required
    nogtis = .false.
    if (booleanParameter("gtis")) then
      GTI_File = StringParameter("gtiset")
      if (setExists(GTI_File)) then
         Set_gti = dataSet(GTI_File,read)
         Table_gti = table(Set_gti,'STDGTI',read) 
         Col_Start = Column(Table_gti,'START',read)
         nrow_gti = numberOfRows(Col_Start)
!         if (nrow_gti > 0) then
           commgtis = gti2xronwin // '"'// trim(GTI_File)// '" '// ' -o win.wi  > temp.txt'
           call system(commgtis)
           call system('rm -f temp.txt')
         if (nrow_gti < 1) then
           nogtis = .true. 
           call warning('EFFTPLOT','NoGTIs')     
         end if
      else 
        call error('EFFTPLOT','GTI file does not exist: '//GTI_File)     
      end if
    end if

    ! Get the efftplot parameters and the default parameters for powspec
    call get_str_param(listoptions,nbopt,maxopt,nbusrpara)

    ! Get information about the input file :
    ! nb_pt : number of bins in the lc
    ! nb_col : number of columns in the RATE extension
    ! l_name_col : name of each column
    ! l_nb_sub_col : number of ``subcolumns'' for each column
    call get_infos(Input_File,Set,nb_pt,nb_col,l_name_col,l_nb_sub_col)  
    

    ! Test of the nbint value (number of bins per interval : a power of 2, less than nb_pt)
    ! Determine nintfm (the number of interval per frame) : the final spectrum must contain only
    ! one averaged spectrum. 

    call calc_param(listoptions,'nbint','nintfm',nb_pt)
    
    Output_File = StringParameter('outfile')
    Device_type = value_option('plotdev',listoptions,var1)

    call open_plot(Device_type, Output_File)
!    call create_labels(Input_File,Set,Label)

    ! For each column of the RATE extension
    do col=1,nb_col
       if (l_name_col(col)(1:4) == 'RATE' ) then
          ! For each ``subcolumns''
          do subcol = 1,l_nb_sub_col(col)
             ! Construction of the powspec command line
             c1 = achar(48+col)
             v1 = achar(48+subcol)
             opt1 = ' vy' // c1 // ' fe' // v1 // ' le' // v1
             call make_cmd_opt(opt2,listoptions,subcol)
             Commande = powspec // '"' // Input_File(1:len_trim(Input_file)) &
             // opt1(1:len_trim(opt1)) // '" ' // opt2(1:len_trim(opt2)) // &
             ' > efftplot.tempo'
             ! run powspec
             call system(Commande)
             ! delete powspec output
             call system('rm efftplot.tempo')
             ! read in the fits file containing the PDS
              call read_pds(Freq,Pds,Pds_Err,Size_Vectors,subcol,nogtis,Output_File)

          if (Size_Vectors <= 0) cycle

             ! plot it
             call draw_plot(Label, INSTRUME, Freq, Pds , Pds_Err, &
                                       Size_Vectors, subcol,listoptions)
          end do
          if (nogtis) then
             Commande = rm  // '"' // trim(Output_File) // '"'
             call system(Commande)
          end if 
         ! delete all the PDS fits files
          call system('rm fout_pds*')
          call system('rm -f temp.fit')
          call system('rm -f win.wi')
       end if
    end do    
    call pgend

    deallocate(l_nb_sub_col, Freq, Pds, Pds_Err)

  end subroutine efftplot


!----------------------------------------------------------------
  subroutine get_str_param(listopt, nbopt, maxopt, nbusrpara)
!----------------------------------------------------------------

! Makes a 2d string array with containing the options for powspec
! and 2 plot options (x_scale and y_scale)
! The format is : (name of option , value of option).

    character(len=*),dimension(2,*),intent(out)  :: listopt
    integer,intent(out) :: nbopt
    integer,intent(in) :: maxopt,nbusrpara

    ! The first 12 parameters are efftplot parameters
    ! The remaining 24 are specific powspec parameters with default values

    character(len=15),dimension(12) :: allopttoread = &
         (/ "nbins          ","rebintype      ","polyorder      ", &
            "operation      ","plotdevice     ","outfile        ", &
            "npoints        ", & 
            "nerrorpoints   ","normalization  ", &
            "xscale         ","yscale         ", &
            "gtiset         " /)    

    character(len=15),dimension(36) :: allopt = &
         (/ "nbint          ","rebin          ","trend_degree   ", &
            "trend_oper     ","device_type    ","outfile        ", &
            "gapfill        ", & 
            "errorbars      ","normalization  ", &
            "x_scale        ","y_scale        ", &
            "window         ",&    
            "outfiletype    ","plot           ","nintfm         ",&
            "tchat          ","lchat          ","logname        ", &
            "clobber        ","dpath          ","plotfile       ", &
            "forcestart     ","exposure       ", &
            "outfileroot    ","simultaneous   ","spwinbefore    ", &
            "spwinafter     ","dtnb           ", &
            "rescale        ","offset         ","fast           ", &
            "nser           ","ipow2          ","iavgreb        ", &
            "nbdf           ","mode           " /)
    
    character(len=15),dimension(24) :: valopt = &
         (/ "2              ","no             ","1              ",&
            "10             ","0              ","xronos.log     ", &
            "YES            ","XRDEFAULTS     ","-              ", &
            "no             ","no             ", &
            "default        ","no             ","0.0            ", &
            "0.0            ","-1             ", &
            "1.0            ","0.0            ","yes            ", &
            "1              ","1              ","0              ", &
            "8192           ","ql             " /)

    character(len=128) :: str 
    character(len=15) :: tmp 
    integer :: indic,i,opt_int,err
    real(kind=single) :: opt_real,opt_real_int


    indic = 0
    nbopt = 0 
        
    do indic=1,nbusrpara-1
       tmp = allopttoread(indic)
       select case(indic)
          case(1,3,4,7,8,9)
             opt_int = intParameter(tmp(1:len_trim(tmp))) 
             str = int2asc(opt_int) 
          case(2,5,6,10,11,12) 
             str =  change_case(stringParameter(tmp(1:len_trim(tmp))),2)
          end select
       nbopt = nbopt+1
       listopt(1,indic) = allopt(indic)
       listopt(2,indic) = str
    end do


    if (.not. booleanParameter("gtis")) then 
       listopt(2,12) = "-"
    else
       listopt(2,12) = "win.wi"
    end if
    if (.not. booleanParameter("removetrend")) then
       listopt(2,3) = "0"
       listopt(2,4) = "1"
    end if
    if (.not. booleanParameter("rebin")) then
        listopt(2,2) = "0"
    else
        read(listopt(2,2),*,iostat=err) opt_real
        if (err/=0) then
            listopt(2,2) = "0"
            call warning('EFFTPLOT','Invalid rebin option: No rebinning')
        else
            if (opt_real >= 0.0) then
               opt_real_int = anint(opt_real)
               if (opt_real /= opt_real_int) then
                 opt_real = opt_real_int
                 call warning('EFFTPLOT','Invalid positive rebin: Set to nearest integer')
               endif
            else
               if ((opt_real >= -1.0).or.(opt_real < -3.0)) then
                 opt_real = 0.0
                 call warning('EFFTPLOT','Rebin is out of range: No rebinning')
               endif
            end if
            write(listopt(2,2),'(F7.4)') opt_real
        end if
    end if
    if (.not. booleanParameter("fillgaps")) listopt(2,7) = "0"   
    listopt(1,3) = "itre"
    listopt(1,4) = "itremo"
    listopt(1,5) = "plotdev"

    if ((listopt(2,10) /= "LOG") .and. (listopt(2,10) /= "LIN")) then
        call warning('EFFTPLOT','Invalid xscale option : set to LIN')
        listopt(2,10) = "LIN"
    end if 
    if ((listopt(2,11) /= "LOG") .and. (listopt(2,11) /= "LIN")) then
        call warning('EFFTPLOT','Invalid yscale option : set to LIN')
        listopt(2,11) = "LIN"
    end if 

    if (listopt(2,10) == "LOG") then; listopt(2,10)="0";end if 
    if (listopt(2,10) == "LIN") then; listopt(2,10)="1";end if 
    if (listopt(2,11) == "LOG") then; listopt(2,11)="0";end if
    if (listopt(2,11) == "LIN") then; listopt(2,11)="1";end if
    
    i = 0
    do indic=nbusrpara, maxopt
       i = i + 1
       tmp = allopt(indic)
       nbopt = nbopt+1
       listopt(1,indic) = tmp
       listopt(2,indic) = valopt(i)
    end do

                  
  end subroutine get_str_param 

  !-------------------------------------------------------------------------------------
  subroutine create_labels(Input_File,Ts_Set , Label, INSTRUME)
  !-------------------------------------------------------------------------------------
  ! Generates labels to add on the plot.

    type( dataSetT ),intent(in) :: Ts_set
    character(len=1000),dimension(8),intent(out) :: Label
    character(len=4) :: offset_units, ra_hr_ch,ra_min_ch,dec_deg_ch, dec_min_ch
    character(len=12) :: TELESCOP, INSTRUME, DETNAM, DATAMODE, OBSERVER
    character(len=12) :: OBJECT, OBS_ID, SRC_CRA, SRC_CDEC
    character(len=12) :: DATE_END, EXPOSURE, TIMEZERO, TIMEDEL, EXP_ID, SRCNUM
    character(len=30) :: TSTART, TSTART_MJDINT, TSTART_MJDDEC, DATE_OBS, RA
    character(len=30) :: DEC, ra_sec_ch, dec_sec_ch
    character(len=128) :: Input_File
    character(len=8) :: fparkey = 'fparkey '
    character(len=512) :: Command
    real(kind=double) :: TIME, TIME_OFFSETINT, TIME_OFFSETDEC, ra_dec, dec_dec
    integer :: Length, ra_min, dec_deg, dec_min, ra_hr
    real(kind=double) :: mjd, mjd0, TIME_OFFSET,timescale_offset,ra_sec,dec_sec
    real              :: dj, rsec
    integer           :: year, month, day, hour, min, sec, status

    type( TableT ) :: Table1 

    Table1 = table(Ts_Set,BlockNumber(Ts_Set,'RATE')) 

!    if (HasAttribute(Ts_Set,"OBSSTART") ) then
!      Command = fparkey // 'dummy ' // trim(Input_File) // '[0] -OBSSTART ' 

!      call system(Command) 
!    endif
!    if (HasAttribute(Ts_Set,"OBSSTOP") ) then
!      Command = fparkey // 'dummy ' // trim(Input_File) // '[0] -OBSSTOP ' 

!     call system(Command) 
!    endif

!    if (HasAttribute(Table1,"EXPOSURE") ) then
!      Command = fparkey // 'dummy ' // trim(Input_File) // '[1] -EXPOSURE ' 

!     call system(Command)
!    endif

    if (HasAttribute(Ts_Set,"SRCNUM") ) then
         SRCNUM = StringAttribute(Ts_Set,'SRCNUM')
    else if (HasAttribute(Table1,"SRCNUM") ) then
         SRCNUM = StringAttribute(Table1,'SRCNUM')
    else
         SRCNUM = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"SRC_CRA") ) then
         RA = StringAttribute(Ts_Set,'SRC_CRA')
    else if (HasAttribute(Table1,"SRC_CRA") ) then
         RA = StringAttribute(Table1,'SRC_CRA')
    else
         RA = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"SRC_CRA") ) then
      ra_dec = real64Attribute(Ts_Set,'SRC_CRA')
      ra_hr = int(ra_dec/15.0)
      write(ra_hr_ch,'(I2)') ra_hr
      ra_min = int(((ra_dec/15.0)-ra_hr)*60.0)
      write(ra_min_ch,'(I2)') ra_min
      ra_sec = ((((ra_dec/15.0)-ra_hr)*60.0)-ra_min)*60.0
      if (ra_sec < 10.0) then
        call string_of_real2(ra_sec, ra_sec_ch, Length)
        write(ra_sec_ch,'(A4)') ra_sec_ch
      else
        call string_of_real2(ra_sec, ra_sec_ch, Length)
        write(ra_sec_ch,'(A5)') ra_sec_ch
      end if
    else if (HasAttribute(Table1,"SRC_CRA") ) then
      ra_dec = real64Attribute(Table1,'SRC_CRA')
      ra_hr = int(ra_dec/15.0)
      write(ra_hr_ch,'(I2)') ra_hr
      ra_min = int(((ra_dec/15.0)-ra_hr)*60.0)
      write(ra_min_ch,'(I2)') ra_min
      ra_sec = ((((ra_dec/15.0)-ra_hr)*60.0)-ra_min)*60.0
      if (ra_sec < 10.0) then
        call string_of_real2(ra_sec, ra_sec_ch, Length)
        write(ra_sec_ch,'(A4)') ra_sec_ch
      else
        call string_of_real2(ra_sec, ra_sec_ch, Length)
        write(ra_sec_ch,'(A5)') ra_sec_ch
     end if     
   else
      ra_hr_ch = '??' ;  ra_min_ch = '??'; ra_sec_ch = '??'
   end if

    if (HasAttribute(Ts_Set,"SRC_CDEC") ) then
         DEC = StringAttribute(Ts_Set,'SRC_CDEC')
    else if (HasAttribute(Table1,"SRC_CDEC") ) then
         DEC = StringAttribute(Table1,'SRC_CDEC')
    else
         DEC = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"SRC_CDEC") ) then
      dec_dec = real64Attribute(Ts_Set,'SRC_CDEC')
      dec_deg = int(dec_dec)
      write(dec_deg_ch,'(I3)') dec_deg
      dec_min= int(abs(((dec_dec)-dec_deg)*60.0))
      write(dec_min_ch,'(I2)') dec_min
      dec_sec = (abs(((dec_dec)-dec_deg)*60.0)-dec_min)*60.0
      if (dec_sec < 10.0) then
        call string_of_real2(dec_sec, dec_sec_ch, Length)
        write(dec_sec_ch,'(A3)') dec_sec_ch
      else
        call string_of_real2(dec_sec, dec_sec_ch, Length)
        write(dec_sec_ch,'(A4)') dec_sec_ch
      end if      
    else if (HasAttribute(Table1,"SRC_CDEC") ) then
      dec_dec = real64Attribute(Table1,'SRC_CDEC')
      dec_deg = int(dec_dec)
      write(dec_deg_ch,'(I3)') dec_deg
      dec_min= int(abs(((dec_dec)-dec_deg)*60.0))
      write(dec_min_ch,'(I2)') dec_min
      dec_sec = (abs(((dec_dec)-dec_deg)*60.0)-dec_min)*60.0
      if (dec_sec < 10.0) then
        call string_of_real2(dec_sec, dec_sec_ch, Length)
        write(dec_sec_ch,'(A3)') dec_sec_ch
      else
        call string_of_real2(dec_sec, dec_sec_ch, Length)
        write(dec_sec_ch,'(A4)') dec_sec_ch
      end if      
    else
      dec_deg_ch = '??' ;  dec_min_ch = '??'; dec_sec_ch = '??'
    end if

    if (HasAttribute(Ts_Set,"TSTART") ) then
         TIME = real64Attribute(Ts_Set,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else if (HasAttribute(Table1,"TSTART") ) then
         TIME = real64Attribute(Table1,'TSTART')
         call string_of_real2(TIME, TSTART, Length)
    else
         TSTART = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Table1,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Table1,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

! Here we read the input information to determine whether an offset TSTART
! is required or not.  If it is required, the offset and its units are also 
! read and the value is passed as a real and a character for plotting.

    if (TSTART /= 'Indef') then
      TIME_OFFSET = 50814.0 + TIME/86400.0 
      TIME_OFFSETINT=int(TIME_OFFSET)
      TIME_OFFSETDEC=TIME_OFFSET-int(TIME_OFFSET)
      call string_of_real2(TIME_OFFSETINT, TSTART_MJDINT, Length)
      call string_of_real2(TIME_OFFSETDEC, TSTART_MJDDEC, Length)
    end if
!    else if (TSTART /= 'Indef' .and. booleanParameter("offset")) then
!      offset_units = StringParameter("ounits")
!      timescale_offset = RealParameter("tscaleoffset")
!      if (timescale_offset == 0.0) then
!        if (len_trim(DATE_OBS) < 19) then
!          write(errstr,*) "Bad format DATE-OBS keyword ", trim(DATE_OBS),  &
!                       " in ", trim(qualifiedName(Table1)),  &
!                       ". Should be yyyy-mm-ddThh:mm:ss"
!          call warning("getXmmDate13")
!          return
!        endif
!        if (DATE_OBS( 5: 5) /= "-" .or. DATE_OBS( 8: 8) /= "-" .or.  &
!          DATE_OBS(11:11) /= "T" .or.  &
!          DATE_OBS(14:14) /= ":" .or. DATE_OBS(17:17) /= ":") then
!          write(errstr,*) "Bad format DATE-OBS keyword ", trim(DATE_OBS),  &
!                       " in ", trim(qualifiedName(Table1)),  &
!                       ". Should be yyyy-mm-ddThh:mm:ss"
!          call warning("getXmmDate13")
!          return
!        endif
!        read(DATE_OBS,100) year, month, day, hour, min, sec
!100     format(I4,1x,I2,1x,I2,1x,I2,1x,I2,1x,I2)
!        call sla_cldj(1998,01,01,mjd0,status)
!        call sla_cldj(year,month,day,mjd,status)
!        if (status > 0) then
!          write(errstr,*) "Error", status, " in sla_cldj for DATE-OBS = ",  &
!          trim(DATE_OBS), " in ", trim(qualifiedName(Table1)),  &
!          ". Should be yyyy-mm-ddThh:mm:ss"
!          call warning("getXmmDate13")
!          return
!        endif
!        rsec = sec
!        call sla_ctf2d(hour,min,rsec,dj,status)
!        if (status > 0) then
!          write(errstr,*) "Error", status, " in sla_ctf2d for DATE-OBS = ",  &
!          trim(DATE_OBS), " in ", trim(qualifiedName(Table1)),  &
!          ". Should be yyyy-mm-ddThh:mm:ss"
!          call warning("getXmmDate13")
!          return
!        endif
!        timescale_offset=mjd - mjd0 + dj + 50814.0
!        if (offset_units == 'secs') then
!          timescale_offset=timescale_offset*86400.0
!        end if
!      end if
!
!      if (offset_units == 'secs') then
!        TIME_OFFSET = timescale_offset/86400.0 
!        TIME_OFFSETINT=int(TIME_OFFSET)
!        TIME_OFFSETDEC=TIME_OFFSET-int(TIME_OFFSET)
!        call string_of_real2(TIME_OFFSETINT, TSTART_MJDINT, Length)
!        call string_of_real2(TIME_OFFSETDEC, TSTART_MJDDEC, Length)
!      end if
!      if (offset_units == 'days') then
!        TIME_OFFSET = timescale_offset
!        TIME_OFFSETINT=int(TIME_OFFSET)
!        TIME_OFFSETDEC=TIME_OFFSET-int(TIME_OFFSET)
!        call string_of_real2(TIME_OFFSETINT, TSTART_MJDINT, Length)
!        call string_of_real2(TIME_OFFSETDEC, TSTART_MJDDEC, Length)
!      end if
!    end if

    if (HasAttribute(Ts_Set,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Ts_Set,'TIMEDEL')
    else if (HasAttribute(Table1,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Table1,'TIMEDEL')
    else
         TIMEDEL = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TIMEZERO") ) then
         TIMEZERO = StringAttribute(Ts_Set,'TIMEZERO')
    else if (HasAttribute(Table1,"TIMEZERO") ) then
         TIMEZERO = StringAttribute(Table1,'TIMEZERO')
    else
         TIMEZERO = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Set,'EXPOSURE')
    else if (HasAttribute(Table1,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Table1,'EXPOSURE')
    else
         EXPOSURE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Set,'DATE-END')
    else if (HasAttribute(Table1,"DATE-END") ) then
         DATE_END = StringAttribute(Table1,'DATE-END')
    else
         DATE_END = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Table1,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Table1,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Set,'OBS_ID')
    else if (HasAttribute(Table1,"OBS_ID") ) then
         OBS_ID = StringAttribute(Table1,'OBS_ID')
    else
         OBS_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Set,'OBJECT')
    else if (HasAttribute(Table1,"OBJECT") ) then
         OBJECT = StringAttribute(Table1,'OBJECT')
    else
         OBJECT = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Set,'OBSERVER')
    else if (HasAttribute(Table1,"OBSERVER") ) then
         OBSERVER = StringAttribute(Table1,'OBSERVER')
    else
         OBSERVER = 'Indef'
    endif

   if (HasAttribute(Ts_Set,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Set,'DATAMODE')
    else if (HasAttribute(Table1,"DATAMODE") ) then
         DATAMODE = StringAttribute(Table1,'DATAMODE')
    else
         DATAMODE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Set,'DETNAM')
    else if (HasAttribute(Table1,"DETNAM") ) then
         DETNAM = StringAttribute(Table1,'DETNAM')
    else
         DETNAM = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Set,'INSTRUME')
    else if (HasAttribute(Table1,"INSTRUME") ) then
         INSTRUME = StringAttribute(Table1,'INSTRUME')
    else
         INSTRUME = 'Indef'
    endif

    if (trim(INSTRUME) == 'EPN') then
      INSTRUME = 'PN'
    else if (trim(INSTRUME) == 'EMOS1') then
      INSTRUME = 'M1'
    else if (trim(INSTRUME) == 'EMOS2') then
      INSTRUME = 'M2'
    else
         INSTRUME = INSTRUME
    endif

    if (HasAttribute(Ts_Set,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Set,'TELESCOP')
    else if (HasAttribute(Table1,"TELESCOP") ) then
         TELESCOP = StringAttribute(Table1,'TELESCOP')
    else
         TELESCOP = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPIDSTR") ) then
         EXP_ID = StringAttribute(Ts_Set,'EXPIDSTR')
    else
         EXP_ID = 'Indef'
    endif

!    Label(1) = trim(TELESCOP)//' - '// &
!               trim(INSTRUME)//' - '// &
!               trim(DETNAM)//' - '// &
!               trim(DATAMODE)// &
!               ' - PI NAME : '//trim(OBSERVER)// &
!               ' - OBJECT : '//trim(OBJECT)// &
!               ' - OBS_ID : '//trim(OBS_ID)
!    Label(2) = 'START : '//trim(DATE_OBS)//' - '// &
!               'STOP : '//trim(DATE_END)
!    Label(3) ='SOURCE EXPOSURE TIME='//trim(EXPOSURE)//' secs '&
!         //' - TIME ZERO : '//trim(TIMEZERO)//' MJD' &
!         //' - BIN TIME : '//trim(TIMEDEL)//' secs'

    Label(1) = 'Filename: '//trim(Input_File)
    Label(2) = 'Obs. ID: '//trim(OBS_ID)//' ('//trim(OBJECT)// ')'
    Label(3)= 'Exp ID: '//trim(INSTRUME)//trim(EXP_ID)
    Label(4) ='Source no.: '//trim(SRCNUM)
    Label(5) ='Source RA_corr: '//trim(ra_hr_ch)//':'//trim(ra_min_ch)//':'//trim(ra_sec_ch)//' (J2000)'
    Label(6) ='Source Dec_corr: '//trim(dec_deg_ch)//':'//trim(dec_min_ch)//':'//trim(dec_sec_ch)//' (J2000)'
    Label(7) = 'Time series bin size: '//trim(TIMEDEL)//' secs'
    Label(8) = 'Time series start (MJD): '//trim(TSTART_MJDINT)//TSTART_MJDDEC(2:10)

  end subroutine create_labels

  !-------------------------------------------------------------------------------------
  subroutine make_cmd_opt(opt2, listoptions, pltnb)
  !-------------------------------------------------------------------------------------
  ! construct the option command line
    character(len=512),intent(out) :: opt2
    character(len=15),dimension(:,:),intent(in) :: listoptions
    integer,intent(in) :: pltnb

    character(len=15) :: tmpst1,tmpst2
    integer :: maxopt,i

    maxopt = size(listoptions,2)
    tmpst1 = listoptions(1,1)
    tmpst2 = listoptions(2,1)
    opt2 = tmpst1(1:len_trim(tmpst1)) // '=' // tmpst2(1:len_trim(tmpst2))
    do i=2,maxopt
       tmpst1 = listoptions(1,i)
       if (tmpst1(2:len_trim(tmpst1)) /= '_scale') then 

          tmpst2 = listoptions(2,i)
          if (tmpst1=='outfile       ') tmpst2 = "fout_pds_" // achar(48+pltnb)
          opt2 = opt2(1:len_trim(opt2)) // ' ' // tmpst1(1:len_trim(tmpst1)) // '=' // &
                 tmpst2(1:len_trim(tmpst2))          
       end if
    end do


  end subroutine make_cmd_opt


  !-------------------------------------------------------------------------------------
  subroutine get_infos(fich, Set, nb_pt, nb_col, l_name_col, l_nb_subcol)
  !-------------------------------------------------------------------------------------
  ! Get information in fits table.

    character(len=*),intent(in) :: fich
    character(len=*),dimension(:) :: l_name_col
    integer,dimension(:),pointer :: l_nb_subcol
    integer,intent(out) :: nb_pt,nb_col
    type( DataSetT ),intent(out) :: Set

    type( TableT ) :: Table1 
    type (ColumnT) :: Col_t
    integer :: i, j
    integer,dimension(:), pointer :: tmp
    real :: Eps
    

    Set = DataSet(fich,READ)
    Table1 = table(Set,BlockNumber(Set,'RATE')) 

    ! Check the validy of extension
    if ( .not. HasAttribute(Table1,"HDUCLASS") ) &
      call warning('EFFTPLOT','Keyword HDUCLASS missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLASS"),2) /= 'OGIP' ) &
      call warning('EFFTPLOT','HDUCLASS is not equal to OGIP')
    if ( .not. HasAttribute(Table1,"HDUCLAS1") ) &
      call warning('EFFTPLOT','Keyword HDUCLAS1 missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLAS1"),2) /= 'LIGHTCURVE' ) &
      call warning('EFFTPLOT','HDUCLAS1 is not equal to LIGHTCURVE')
    if ( .not. HasAttribute(Table1,"HDUCLAS2") ) &
      call warning('EFFTPLOT','Keyword HDUCLAS2 missing in the input file '//fich)
!    if ( change_case(StringAttribute(Table1,"HDUCLAS2"),2) /= 'TOTAL' ) &
!      call warning('EFFTPLOT','HDUCLAS2 is not equal to TOTAL')
    if ( .not. HasAttribute(Table1,"HDUCLAS3")) &
      call warning('EFFTPLOT','Keyword HDUCLAS3 missing in the input file '//fich)
    if ( change_case(StringAttribute(Table1,"HDUCLAS3"),2) /= 'RATE' ) &
      call warning('EFFTPLOT','HDUCLAS3 is not equal to RATE')

    Eps = epsilon(0.)
    nb_col = numberOfColumns(Table1)
    nb_pt = numberOfRows(Table1)
    allocate(l_nb_subcol(nb_col))
    do i=0,nb_col-1 
       Col_t = Column(Table1,i,READ)

!    Array_Rate => real32VectorData(Col_Rate)
!    allocate( Net_Rate( size(Array_Rate(:,0)) , size(Array_Rate(0,:)) ) )
!    Net_Rate = Array_Rate 
!       do j=0,nb_pt-1
!         if (Col_t(j) < Eps) then
!           Col_t(j) == 0.0
!         end if
!       end do
       allocate(tmp(size(Dimensions(Col_t))))
       tmp = Dimensions(Col_t)
       l_nb_subcol(i+1) = tmp(1)
       deallocate(tmp)
       l_name_col(i+1) = name(Col_t)
    end do

  end subroutine get_infos


  !----------------------------------------------------------------------------------
  subroutine read_pds(Freq, Pds, Pds_err, Size_Vect, band, nogtis, Output_File)
  !----------------------------------------------------------------------------------
  ! Read pds in powspec output fits file.

    
    real(kind=double),dimension(:),pointer ::tmpd, Freq
    real(kind=single),dimension(:),pointer ::tmps, Pds, Pds_Err
    integer,intent(out) :: Size_Vect
    integer,intent(in) :: band
    integer :: nfreq
    character(len=128) :: Output_File

!    real(kind=double),dimension(:),pointer :: tmp

    character(len=1) :: fillin 
    character(len=8) :: fparkey = 'fparkey '
    character(len=9) :: fmodhead = 'fmodhead '
    character(len=10) :: fkeyprint = 'fkeyprint '
    character(len=128) :: fich
    character(len=512) :: command
    type( DataSetT ) :: Set
    type( TableT ) :: Table1 
    type( ColumnT ) :: Col_Freq, Col_Pds, Col_Pds_Err 
    integer :: nheads, length
    real :: rnhead
    logical :: nogtis

    fich = "fout_pds_" // achar(48+band)
    fich = fich(1:len_trim(fich)) // ".fps"
!   Change the header information of the files that are written out by powspec,
!   so that they are compatible with the SAS software
   

    if( setExists(fich) ) then
      Set = DataSet(fich,READ)
      Table1 = table(Set,"RESULTS") 
      nfreq = numberOfRows(Table1)
      if (nfreq < 4) then
        command = 'rm -f '//Output_File
        call system(command)
        call warning('EFFTPLOT', 'NotEnoughBins')     
        allocate(Freq(0))
        allocate(Pds(0))
        allocate(Pds_Err(0))
        Size_vect = 0
      end if
    else  
      command = 'rm -f '//Output_File
      call system(command)
      call warning('EFFTPLOT', 'NotEnoughBins')     
      allocate(Freq(0))
      allocate(Pds(0))
      allocate(Pds_Err(0))
      Size_vect = 0
  return
    end if  

    if (HasAttribute(Table1,"DATE") ) then
      Command = fkeyprint //  'dummy' // '"' // fich(1:len_trim(fich)) // '[1]' // '"' // ' DATE ' // &
    ' > newstuff.dat '
      call system(Command)
      Command = fmodhead //  'dummy' // '"' // fich(1:len_trim(fich)) // '[0]' // '"' // ' newstuff.dat'
      call system(Command)
    end if
     if (HasAttribute(Table1,"CREATOR") ) then
       Command = fkeyprint //  'dummy' // '"' // fich(1:len_trim(fich)) // '[1]' // '"' // ' CREATOR '// &
    ' > newstuff.dat '
      call system(Command)
      Command = fmodhead //  'dummy' // '"' // fich(1:len_trim(fich)) // '[0]' // '"' // ' newstuff.dat'
      call system(Command)
      call system('rm newstuff.dat')
    end if

!    do nheads=1,2
    do nheads=1,1
      rnhead=real(nheads)
      call string_of_real(rnhead, fillin, length)
      if (HasAttribute(Set,"DATE") ) then
        Command = fparkey // '"' // fich(1:len_trim(fich)) // '[' // &
        fillin // ']' // '"' // ' -DATE '
        call system(Command)
      end if
      if (HasAttribute(Set,"CREATOR") ) then
        Command = fparkey // '"' // fich(1:len_trim(fich)) // '[' // & 
        fillin // ']' // '"' // ' -CREATOR '
        call system(Command)
      end if
    end do   

    allocate(Freq(0:nfreq-1))
    allocate(Pds(0:nfreq-1))
    allocate(Pds_Err(0:nfreq-1))

    Col_Freq = Column(Table1,"FREQUENCY",Read)
    Col_Pds = Column(Table1, "POWER",Read)
    Col_Pds_Err = Column(Table1,"ERROR",Read)

! THIS THE CORRECT WAY TO READ THE SPECTRUM (FIXES SPR-1482 raised by DB)

    Tmps => real32Data(Col_Pds)
    Pds=Tmps
    Tmps => real32Data(Col_Pds_Err)
    Pds_Err=Tmps
    Tmpd => real64Data(Col_Freq)
    Freq=Tmpd

    Size_vect = size(Pds,1)
    call release(Col_Pds)
    call release(Col_Freq)
    call release (Col_Pds_Err)

  end subroutine read_pds

  !-------------------------------------------------------------------------------------
  subroutine open_plot(Device_Type, Output_File) 
  !-------------------------------------------------------------------------------------
  ! Check Device_Type and Output_File. Open plot if required.

    character(len=*),intent(inout) :: Device_Type , Output_File
    integer :: pgbeg
    character(len=2048) :: Device_Selected
    logical :: File_Exists

    if (Output_File == '')  Output_File='default.ps'
    if (Device_Type(1:3) /= '/XW') then
         inquire(file=Output_File,exist=File_Exists)   
         if (File_Exists .and. .not. clobber()) call error('EFFTPLOT', &
         'Already existing file '//trim(Output_File)//' cannot be overwritten (SAS_CLOBBER=0)')
         if (File_Exists .and. clobber()) call warning('EFFTPLOT', &
         'Already existing file '//trim(Output_File)//' will be overwritten')
         Device_Selected=trim(Output_File)//trim(Device_Type)
         if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
             call warning('EFFTPLOT', &
            'Invalid device and/or output file : '//trim(Device_Type)//' , '//trim(Output_File))
             Output_File='default.ps'
             Device_Type= '/PS'
             Device_Selected=trim(Output_File)//trim(Device_Type)
             if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
                  call error('EFFTPLOT', &
                  'Invalid DEFAULT device and/or output file : '//trim(Device_Type)//' , '&
                  //trim(Output_File))
             else
                  call warning('EFFTPLOT', &
                 'New device and output file : '//trim(Device_Type)//' , '//trim(Output_File))
            endif
         endif
    else
        Device_Selected=trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) call error('EFFTPLOT', &
        'Invalid device : '//trim(Device_Type))
   endif
   call pgpap(0.0 , 1.618 )
   call pgask(.true.)
   call pgsch(0.9)

 end subroutine open_plot

 !-------------------------------------------------------------------------------------
  subroutine draw_plot(Label, INSTRUME, Freq0, Pds0, Pds_Err0, Size_Vectors, &
                        nrjbd, listoptions)
 !-------------------------------------------------------------------------------------
 ! Draw a plot which contains the power spectrum (Pds0) vs. (Freq0) with error bars (Pds_Err0)

    character(len=1000),dimension(:),intent(in) :: Label
    character(len=15),dimension(:,:),intent(in) :: listoptions
    character(len=12) :: INSTRUME
    real(kind=double),dimension(:),pointer :: Freq0
    real(kind=single),dimension(:),pointer :: Pds0 , Pds_Err0
    real(kind=double),dimension(:),pointer :: Freq1
    real(kind=single),dimension(:),pointer :: Pds1 , Pds_Err1
    integer,intent(in) :: Size_Vectors,nrjbd

  ! Local variable Window is used to define the viewports (3 figures/page)
    
    real,dimension(3,4) :: Window
    character(len=15) :: tmp,str
    integer :: Energy_Band , i,xsc,ysc,Svect


    Window(1,:) = (/0.09,0.99,0.65,0.83/)
    Window(2,:) = (/0.09,0.99,0.35,0.53/)
    Window(3,:) = (/0.09,0.99,0.05,0.23/)

    Energy_Band = nrjbd
    if (Energy_Band > 3) then 
      if (Energy_Band == 4)  call pgpage
       Energy_Band = Energy_Band -3
    end if
    call pgsvp(0.0,1.0,0.0,1.0)
    call pgsci(1)
!    do i=1,3
!       call pgmtxt('T' , -(1. + (i-1)*1.5) , 0.5 , 0.5 , Label(i))
!    end do
       do i=1,6
!          call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.5 , 0.5 , Label(i))
          call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.05 , 0.0 , Label(i))
       end do
       do i=7,8
!          call pgmtxt('T' , -(1. + (i-1)*1.2) , 0.5 , 0.5 , Label(i))
          call pgmtxt('T' , -(1. + (i-3)*1.2) , 0.55 , 0.0 , Label(i))
       end do
    call pgsci(1)
    call pgsch(0.9)
!    call pgmtxt('T' , -7.*0.5/0.75 , 0.5 , 0.5 , &
!         'EPIC source time series power spectrum')
    call pgsch(0.9)
    tmp = value_option('x_scale',listoptions,i)
    xsc = asc2int(tmp)

    tmp = value_option('y_scale',listoptions,i)
    ysc = asc2int(tmp)

    str = int2asc(nrjbd)
    Svect = Size_Vectors

    if (trim(INSTRUME) == 'PN') then
      INSTRUME = 'PN'
    else if (trim(INSTRUME) == 'M1') then
      INSTRUME = 'MOS 1'
    else if (trim(INSTRUME) == 'M2') then
      INSTRUME = 'MOS 2'
    else
         INSTRUME = INSTRUME
    endif
 
    ! If the y_scale is log then remove the negative Pds0 values 
    ! and plot nonzero values Pds1 vs. Freq1
    ! else plot Pds0 vs. Pds1

    if (ysc == 0) then 
         call del_bad_value(Freq0,Pds0,Pds_Err0,Svect,Freq1,Pds1,Pds_Err1)
         call make_one_plot(str, Window(Energy_Band,:), INSTRUME, Freq1, &
                             Pds1, Pds_Err1, xsc, ysc)
    else
         call make_one_plot(str, Window(Energy_Band,:), INSTRUME, Freq0, &
                             Pds0 , Pds_Err0, xsc, ysc)
    endif

  end subroutine draw_plot

  !-------------------------------------------------------------------------------------
  subroutine make_one_plot(String, Window, INSTRUME, Freq0, Pds0, Pds_Err0, &
                           xsc, ysc)
  !-------------------------------------------------------------------------------------
  ! Draw PDS on Y-axis in function of the frequency (on X-axis).
  ! xsc = 0 ===> X-axis is LOG else X-axis is LIN
  ! ysc = 0 ===> Y-axis is LOG else Y-axis is LIN
  
    character(len=*),intent(in) :: String
    character(len=12),intent(in) :: INSTRUME
    real,dimension(4),intent(in) :: Window
    integer,intent(in) :: xsc,ysc
    real(kind=double),dimension(:),pointer :: Freq0
    real(kind=single),dimension(:),pointer :: Pds0 , Pds_Err0

    real(kind=single),dimension(:),allocatable :: barinf,barsup
    real(kind=single) :: minx,maxx,miny,maxy,fact_y,fact_x,nb_decade
    real(kind=single) :: minimum_value,maximum_value,range

    integer :: i,Size_VEctors

    nb_decade = 0.8
    fact_y = 10 ** nb_decade
    nb_decade = 0.3
    fact_x = 10 ** nb_decade

    Size_VEctors = size(Pds0,1)

    allocate(barinf(Size_VEctors))
    allocate(barsup(Size_VEctors))

    call pgsvp(Window(1),Window(2),Window(3),Window(4))


    if (xsc == 0) then 
       minx = log10(minval(Freq0)/fact_x)
       maxx = log10(maxval(Freq0)*fact_x)
    else 
!       minx = 0.0-minval(Freq0)
       minx = 0.0-(0.025*(maxval(Freq0)))
       maxx = 1.025*(maxval(Freq0))
    endif

    if (ysc == 0) then 

       minimum_value = minval(Pds0)/fact_y
       maximum_value = maxval(Pds0)*fact_y
       miny          = log10(minimum_value)
       maxy          = log10(maximum_value)


       do i=0,size(Pds0,1)-1
          if ( Pds0(i) - Pds_ERR0(i) < minimum_value) then
                barinf(i+1) = minimum_value
          else
                barinf(i+1) = Pds0(i)-Pds_ERR0(i)
          endif
          if ( Pds0(i) + Pds_ERR0(i) > maximum_value) then
                barsup(i+1) = maximum_value
          else
                barsup(i+1) = Pds0(i)+Pds_ERR0(i)
          endif
       end do

    else

       miny  = minval(Pds0-Pds_Err0)
       maxy  = maxval(Pds0+Pds_Err0)
       range = maxy - miny
       miny  = miny - 0.1 * range
       maxy  = maxy + 0.1 * range

       do i=0,size(Pds0,1)-1
           barsup(i+1) = Pds0(i)+Pds_ERR0(i)
           barinf(i+1) = Pds0(i)-Pds_ERR0(i)
       end do

    endif

    call pgswin(minx,maxx,miny,maxy)
    call pgsci(1)
    if (xsc == 0 ) then 
       if (ysc == 0 ) then 
          call pgbox('BCLNST' , 0.0 , 5 ,'BCLNST' , 0.0 , 5)
       else
          call pgbox('BCLNST' , 0.0 , 5 ,'BCNST' , 0.0 , 5)
       endif
    else
       if (ysc == 0 ) then 
          call pgbox('BCNST' , 0.0 , 5 ,'BCLNST' , 0.0 , 5)
       else
          call pgbox('BCNST' , 0.0 , 5 ,'BCNST' , 0.0 , 5)
       endif
    end if
          
!    call pglab('Frequency (Hz)' , 'Power' , 'Rate  '//String)
    call pglab('Frequency (Hz)' , 'Power' , 'EPIC '//trim(INSTRUME)//' source time series power spectrum ')
    call pgsci(2)
!    if (xsc == 1) then
!       if (ysc == 1) then 
!          call pgerry(Size_Vectors , real(Freq0) , barinf , barsup,2.0)
!       else
!          call pgerry(Size_Vectors , real(Freq0) , log10(barinf) , log10(barsup),2.0)
!       end if
!    else
!       if (ysc == 1) then 
!          call pgerry(Size_Vectors, log10(real(Freq0)), barinf, barsup,2.0)
!       else
!          call pgerry(Size_Vectors, log10(real(Freq0)), log10(barinf), log10(barsup),2.0)
!            end if
!    endif
    call pgsci(1)
    call pgslw(1)
    if (xsc == 1) then
       if (ysc == 1) then 
!          call pgpt(Size_Vectors , real(Freq0) , Pds0 , -2)
          call pgline(Size_Vectors-1 , real(Freq0) , Pds0)
       else
!          call pgpt(Size_Vectors , real(Freq0) , log10(Pds0) , -2)
          call pgline(Size_Vectors-1 , real(Freq0) , log10(Pds0))
       end if
    else
       if (ysc == 1) then 
!          call pgpt(Size_Vectors , log10(real(Freq0)) , Pds0 , -2)
          call pgline(Size_Vectors-1 , log10(real(Freq0)) , Pds0)
       else
!          call pgpt(Size_Vectors , log10(real(Freq0)) , log10(Pds0) , -2)
          call pgline(Size_Vectors-1 , log10(real(Freq0)) , log10(Pds0))
       end if
    end if
    call pgslw(1)

    deallocate(barinf)
    deallocate(barsup)

  end subroutine make_one_plot

  !-------------------------------------------------------------------------
  subroutine del_bad_value(Freq0,Pds0,Pds_Err0,Size_VEctors,Freq,Pds,Pds_Err) 
  !-------------------------------------------------------------------------
  ! This subroutine eliminates the negative values of Pds0
  ! On ouput, Pds contains only positive values

    real(kind=single),dimension(:),pointer :: Pds0,Pds_Err0
    real(kind=double),dimension(:),pointer :: Freq0
    integer,intent(inout) :: Size_VEctors

    real(kind=double),dimension(:),pointer :: Freq
    real(kind=single),dimension(:),pointer :: Pds,Pds_Err


    integer :: nb,i,j

    nb=0
    do i=0,size(Freq0,1)-1
       if (Pds0(i)  > 0.) nb =nb +1
    end do

    allocate(Freq(0:nb-1))
    allocate(Pds(0:nb-1))
    allocate(Pds_Err(0:nb-1))

    j=0
    do i=0,size(Freq0,1)-1
       if (Pds0(i)  > 0.) then 
          Freq(j) = Freq0(i)
          Pds(j) = Pds0(i)
          Pds_Err(j) = Pds_Err0(i)
          j = j +1
       end if
    end do
    Size_VEctors = nb

  end subroutine del_bad_value

  !-------------------------------------------------------------------------------------
  subroutine calc_param(listeopt,nbintpara,nintfmpara,Nb_data)
  !-------------------------------------------------------------------------------------
  ! Check and modify if necessary the nbint value
  ! Calculate the value of nintfm 

    character(len=*),dimension(:,:),intent(inout)  :: listeopt
    character(len=*),intent(in) :: nbintpara,nintfmpara
    integer :: Nb_data,nbint_test
     
    character(len=15) :: nbint_str,nintfm_str
    integer :: rank,nbint,nintfm
    
    nbint_str = value_option(nbintpara,listeopt,rank)
    nbint = asc2int(nbint_str)

    if (nbint==1) then 
       ! Full resolution spectrum
       nbint = 2**aint((log(real(Nb_data))/log(2.0)))
       listeopt(2,rank) = int2asc(nbint)
    else
       if (nbint>Nb_data) then 
          ! Full resolution spectrum and warning
          nbint = nint(2.0**aint((log(real(Nb_data))/log(2.0))))
          listeopt(2,rank) = int2asc(nbint)
          call warning('EFFTPLOT', & 
                'Not enough bins : '//trim(int2asc(Nb_data))//' data bins.'//&
                ' nbint set to : '// listeopt(2,rank))
       else
          ! Verify if nbint is a power of 2
          nbint_test = 2**anint((log(real(nbint))/log(2.0)))
          if (nbint_test>Nb_data) nbint_test = 2**aint((log(real(nbint))/log(2.0))) 
          if (nbint_test == nbint) then 
              listeopt(2,rank) = int2asc(nbint)
          else
              listeopt(2,rank) = int2asc(nbint_test)
               call warning('EFFTPLOT', & 
                'nbint is set to the closest power of 2 : '// listeopt(2,rank))
          endif
       end if
    end if
    
    ! Set the value of nintfm

    nintfm_str = value_option(nintfmpara,listeopt,rank)
    nintfm = asc2int(nintfm_str)
! To correct for SPR-1564; the last incomplement segment is removed
!    nintfm = 1 + Nb_data /nbint 
    nintfm = Nb_data /nbint 
    listeopt(2,rank) = int2asc(nintfm)

  end subroutine calc_param

  !-------------------------------------------------------------------------------------
  subroutine string_of_real(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=single),intent(in) :: Nb
    character(len=1),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 6 , 6-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real

  !-------------------------------------------------------------------------------------
  subroutine string_of_real2(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=double),intent(in) :: Nb
    character(len=30),intent(out) :: String
    integer,intent(out) :: Length
    integer:: puiss
    integer::MM

    puiss = max( 0 , min( 8 , 8-int(log10(Nb)) ) )
    MM = idnint(Nb*real(10**puiss))
    call pgnumb(MM , -puiss , 1 , String , Length)

  end subroutine string_of_real2

  !-------------------------------------------------------------------------------------
  function asc2int(str) result(nb)
  !-------------------------------------------------------------------------------------
    character(len=*),intent(in) :: str
    integer :: nb

    integer :: i,lg

    nb = 0
    lg = len_trim(str)
    do i=1,lg
       nb = nb + (10**(lg-i)) * (iachar(str(i:i))-48)
    end do
       
  end function asc2int

  !-------------------------------------------------------------------------------------
  function int2asc(nb0) result(str)
  !-------------------------------------------------------------------------------------
    integer,intent(in) :: nb0
    character(len=15) :: str

    integer :: i,lg,cp,lenstr,posi,deb,nb
    character(len=1) :: tmp
    
    deb = 1
    nb =nb0
    str = "               "
    if (nb == 0) then 
          str(1:1) = "0"
    else
       if (nb < 0) then 
          deb = 2
          str(1:1) = "-"
          nb = -nb
       end if
       lg = aint(log10(real(nb))) + deb
       lenstr = len(str)
       cp = nb
       do i=1,lg 
          tmp = achar(48+mod(cp,10))
          posi = lg-i+deb
          str(posi:posi) = tmp
          cp = cp / 10
       end do     
    end if
  end function int2asc

  !-------------------------------------------------------------------------------------
  function value_option(option,instringtab,rank) result(outstring)
  !-------------------------------------------------------------------------------------
  ! This function returns the value of an 'option' in the table 'instringtab' 

    character(len=*),dimension(:,:),intent(in) :: instringtab
    character(len=*),intent(in) :: option
    character(len=len(option)) :: outstring
    integer,intent(inout) :: rank

    character(len=len(option)) :: tmp
    integer :: i
    
    do i =1, size(instringtab,2) 
       tmp = instringtab(1,i)
       if (tmp(1:len_trim(tmp))==option(1:len_trim(option)))  then 
          outstring = instringtab(2,i)
          rank = i
       end if
    end do
    end function value_option


  !-------------------------------------------------------------------------------------
  function change_case(instring,dir) result(outstring)
  !-------------------------------------------------------------------------------------
  ! Switch from lower to upper case or vice-versa: From Jean Ballet
  ! instring : input string
  ! dir      : direction (1: to lower case; 2: to uppercase; 0: switch case)

    character(len=*), intent(in) :: instring
    integer, intent(in) :: dir
    character(len=len(instring)) :: outstring

  ! ASCII positions of lower case and upper case letters

    integer, parameter :: start_up  = iachar("A"), end_up  = iachar("Z"), &
                          start_low = iachar("a"), end_low = iachar("z"), &
                          offset = start_low - start_up

    integer :: length, icur, pos

    length = len(instring)
    outstring = instring
    do icur = 1, length
       pos = iachar(instring(icur:icur))
       if (pos >= start_up .and. pos <= end_up .and. dir < 2) then
          pos = pos + offset
       else if (pos >= start_low .and. pos <= end_low .and. dir /= 1) then
          pos = pos - offset
       endif
       outstring(icur:icur) = achar(pos)
    enddo

  end function change_case  
  

end module efftplot_mod


